######################################
# EN RECURSIF
######################################

def calculLabyRecursif(laby,x,y):
    """
    effectue un calcul de distance en recursif
    """
    # faire un nouveau tableau avec 10000
    max = 1000
    tab=[]
    for i in range(len(laby)):
        t=[]
        for j in range(len(laby[0])) :
            t+=[max]
        tab+=[t]

    # appel recursif propager
    propagerValeur(tab,laby,x,y,0)
    return tab


def propagerValeur(tab, laby, x, y, indice,max=1000,PRINT=True):
    """
    appel recursif pour propager la valeur
    - si deja une meilleure valeur, rien a faire
    - sinon, tagguer la case et repropager aux voisins
    """

    # si deja taggue plus petit
    if PRINT :
        print ("-> traite "+str(x)+","+str(y)+" valeur: "+str(indice), end=' ')

    if tab[x][y]<indice :
        # LOG
        if PRINT :
            print ("=> deja mieux "+str(tab[x][y]))
        # stop
        return

    if PRINT :
        if (tab[x][y] == max):
            print ("=> nouvelle case")
        else :
            print ("=> amelioration de "+str(tab[x][y]))

    # taggue case
    tab[x][y] = indice
    dessinerChemin(tab)

    # voisins
    voisins=[[0,1],[0,-1],[1,0],[-1,0]]

    # pour chaque voisin
    for v in voisins:
        # voisin
        vx=x+v[0]
        vy=y+v[1]

        # si voisin non mur
        if laby[vx][vy] != 1:
            # appel recursif
            propagerValeur(tab,laby,vx,vy,indice+1)

######################################
# AFFICHAGE DES DISTANCES
######################################

def dessinerChemin(tab):
    """
    permet de dessiner une matrice de distance dans un labyrinthe
    """
    # dessine les lignes
    for j in range(0,len(tab[0])):

        # pour chaque colonnee
        for i in range(0,len(tab)):
            # recupere la valeur calculee
            val=tab[i][j]

            # si c'est 1000, c'est une case non visitee
            if val==1000:
                # affiche vide
                print("   ",end=' ')
            else:
                # affiche la valeur
                print(val,end=' ')
                # ajoute le bon nombre d'espaces en fonction de la taille du nombre
                if val<0:
                    val=-val*10
                if val==0:
                    val=1
                while (val<100):
                    val=val*10
                    print(' ',end='')
        print()

######################################
# CALCUL DU CHEMIN
######################################

def getChemin(distance,depX,depY):
    """
    retourne le chemin a partir d'une matrice de distance
    - depX : abscisse du depart
    - depY : ordoneee du depart
    """
    # valeur maximale
    maxVal=1000

    # on se place au depart
    x=depX
    y=depY
    chemin=[(x,y)]

    # liste des voisins connexite 4
    voisins = [(1,0),(-1,0),(0,1),(0,-1)]

    # tant que 0 est non atteint
    while not distance[x][y] == 0 :
        # cherche plus petite case chez les voisins
        min = maxVal
        voisinMin =()
        # pour chaque voisin
        for voisin in voisins :
            # calcule case voisine
            vx = x + voisin[0]
            vy = y + voisin[1]
            # si meilleure
            if distance[vx][vy] < min :
                min = distance[vx][vy]
                voisinMin = (vx,vy)

        # meilleureVoisin trouve remonte la case
        chemin.append(voisinMin)
        (x,y) = voisinMin

    # fin retourne chemin
    return chemin

######################################
# MAIN
######################################

def main(nameLaby):
    """
    code appele par le __main__
    creer et dessine un Labyrinthe
    """
    import labyrinthe

    # creer un labyrinthe
    laby = labyrinthe.getLabyrinthe(nameLaby)
    xDep,yDep = labyrinthe.getDepart(nameLaby)

    # affiche le labyrinthe dans la console
    labyrinthe.dessinerTableau(laby)

    # calcul distances en recursif
    print("############# RECURSIF ##############")
    tab = calculLabyRecursif(laby,1,1)

    # recupere chemin
    chemin = getChemin(tab,xDep,yDep)
    print (chemin)


######################################
# lancement du module python
######################################


import sys

if __name__ == "__main__":
    # labyrinthe a afficher
    nameLaby=""

    # si trop d'arguments, exit'
    if len(sys.argv) > 2 :
        print ("**ERROR** un seul argument max")
        print ("")
        print ("USAGE :")
        print ("'laby [name]'")
        print ("- avec name désignant le labyrinthe ('default' ou 'grand')")

    # si pas d'argument, labyrinthe defaut
    if len(sys.argv) == 1 :
        nameLaby = "default"
    # sinon arg 1 doit contenir le nom du labyrinthe
    else :
        nameLaby = sys.argv[1]

    main(nameLaby)
